# -*- coding:utf-8 -*-
"""
    author: Li Junxian
    function:read test data xml
"""

import os
import re

from .. import config
from ..function.express.express import Express
from ..function.utils.utils import Utils
from ..function.analysis.xml_parser import XmlParser


class ParserTestData(object):
    __xml_file_re = re.compile(
        r"^test_\d+[a-zA-Z_\u4e00-\u9fa5，。；（）][a-zA-Z_\u4e00-\u9fa5，。；（）0-9]*_\d+[a-zA-Z_\u4e00-\u9fa5，。；（）]["
        r"a-zA-Z_\u4e00-\u9fa5，。；（）0-9]*\.xml$")

    __case_name_re = re.compile(r"^[a-zA-Z0-9_\u4e00-\u9fa5，。；：（）！、,!()+-]+$")

    def __init__(self, test_data_file: str, id_="-1"):
        """
        解析测试数据文件
        :param test_data_file: 文件路径
        :return: None
        """
        # 对传入的文件名进行判断，其格式必须为【test_模块编号模块名称_场景编号场景名称.xml】
        if config.get_config("the_global_inspection",bool) and not self.__xml_file_re.fullmatch(test_data_file):
            raise Exception("案例PY文件的命名必须符合以下规则：\n1、格式[test_模块编号模块名称_场景编号场景名称.py]\n"
                            "2、模块和场景名称只能包含字母数字下划线中文及部分中文符号，且不可以以数字开头；\n"
                            "当前执行的PY文件不符合此规则，请检查对应的案例xml并修改：[{}]".format(test_data_file))
        self.__test_data_file = os.path.join(config.test_data_path, test_data_file)
        self.__test_data = None
        self.__id = id_

    def __parse_xml(self):
        """
        解析测试数据文件
        :return: None
        """
        if not os.path.exists(self.__test_data_file):
            raise Exception("对应的数据xml文件[{}]不存在,请检查[{}]目录".format(self.__test_data_file, config.test_data_path))
        root = XmlParser(self.__test_data_file).parse_xml()
        scene = root["scene"]
        cases = scene["case"]
        cases = cases if type(cases) == list else [cases]
        # 取得db_info
        db_info = Utils.extract_attrs_from_dict(scene, "db_info", "$value")
        # 如果db_info为空，取得全局的
        if db_info is None:
            # 如果全局的不为空，则对其进行计算
            if config.have_config('db_info'):
                db_info = Express.calculate_str(config.get_config('db_info'), {})
        else:
            # 计算db
            db_info = Express.calculate_str(db_info, {})
        real_cases = []
        for case in cases:
            if self.__id != "-1":
                id_ = case.get("id")
                if self.__id != id_:
                    continue
            if db_info is not None and case.get("db_info") is None:
                case["db_info"] = db_info
            elif case.get("db_info") is not None:
                case["db_info"] = Express.calculate_str(case["db_info"], {})
            name = case.get("name")
            if config.get_config("the_global_inspection",bool) and not self.__case_name_re.fullmatch(name) or len(
                    name) > 100:
                raise Exception(
                    "案例名称需符合以下规则：\n1、只能包含字母数字中文。\n2、只能包含中文/英文逗号，中文句号，中文冒号，中文分号，中文/英文括号，中文/英文感叹号，中文分号，加号，减号及下划线。\n3"
                    "、长度不能超过100个字符；\n此案例名：[{}]不符合此规则，请检查！".format(
                        name))
            real_cases.append(case)
        self.__test_data = real_cases

    @property
    def test_data(self):
        if not self.__test_data:
            self.__parse_xml()
        return self.__test_data
